@@ -11,6 +11,17 @@ from fa_purity.frozen import (
     FrozenDict,
     FrozenList,
 )
+from fa_purity.pure_iter.factory import (
+    from_flist,
+)
+from fa_purity.pure_iter.transform import (
+    consume,
+)
+from redshift_client.column import (
+    Column,
+    ColumnId,
+    ColumnObj,
+)
 from redshift_client.data_type.core import (
     DataType,
     PrecisionType,
@@ -174,7 +185,11 @@ class TableClient:
         )
 
     def insert(
-        self, table_id: TableId, table: Table, items: PureIter[RowData]
+        self,
+        table_id: TableId,
+        table: Table,
+        items: PureIter[RowData],
+        limit: int,
     ) -> Cmd[None]:
         _fields = ",".join(f"{{field_{i}}}" for i, _ in enumerate(table.order))
         stm = f"""
@@ -187,7 +202,7 @@ class TableClient:
         for i, c in enumerate(table.order):
             identifiers[f"field_{i}"] = c.name
         return self._db_client.values(
-            dynamic_query(stm, FrozenDict(identifiers)), items
+            dynamic_query(stm, FrozenDict(identifiers)), items, limit
         )
 
     def rename(self, table_id: TableId, new_name: str) -> Cmd[TableId]:
@@ -222,6 +237,37 @@ class TableClient:
     def delete_cascade(self, table_id: TableId) -> Cmd[None]:
         return self._delete(table_id, True)
 
+    def add_column(self, table_id: TableId, column: ColumnObj) -> Cmd[None]:
+        stm = f"""
+            ALTER TABLE {{table_schema}}.{{table_name}}
+            ADD COLUMN {{column_name}}
+            {_encode_data_type(column.column.data_type)} DEFAULT %(default_val)s
+        """
+        identifiers: Dict[str, Optional[str]] = {
+            "table_schema": table_id.schema.name,
+            "table_name": table_id.name,
+            "column_name": column.id_obj.name,
+        }
+        args: Dict[str, PrimitiveVal] = {
+            "default_val": column.column.default,
+        }
+        return self._db_client.execute(
+            dynamic_query(stm, FrozenDict(identifiers)),
+            QueryValues(FrozenDict(args)),
+        )
+
+    def add_columns(
+        self,
+        table: TableId,
+        columns: FrozenDict[ColumnId, Column],
+    ) -> Cmd[None]:
+        return (
+            from_flist(tuple(columns.items()))
+            .map(lambda c: ColumnObj(c[0], c[1]))
+            .map(lambda c: self.add_column(table, c))
+            .transform(lambda x: consume(x))
+        )
+
     def new(
         self, table_id: TableId, table: Table, if_not_exist: bool = False
     ) -> Cmd[None]:
@@ -261,3 +307,47 @@ class TableClient:
         return self._db_client.execute(
             dynamic_query(stm, FrozenDict(identifiers)), QueryValues(values)
         )
+
+    def create_like(self, blueprint: TableId, new_table: TableId) -> Cmd[None]:
+        stm = """
+            CREATE TABLE {new_schema}.{new_table} (
+                LIKE {blueprint_schema}.{blueprint_table}
+            )
+        """
+        identifiers: Dict[str, Optional[str]] = {
+            "new_schema": new_table.schema.name,
+            "new_table": new_table.name,
+            "blueprint_schema": blueprint.schema.name,
+            "blueprint_table": blueprint.name,
+        }
+        return self._db_client.execute(
+            dynamic_query(stm, FrozenDict(identifiers)), None
+        )
+
+    def append(self, source: TableId, target: TableId) -> Cmd[None]:
+        stm = """
+            ALTER TABLE {target_schema}.{target_table}
+            APPEND FROM {source_schema}.{source_table}
+        """
+        identifiers: Dict[str, Optional[str]] = {
+            "source_schema": source.schema.name,
+            "source_table": source.name,
+            "target_schema": target.schema.name,
+            "target_table": target.name,
+        }
+        return self._db_client.execute(
+            dynamic_query(stm, FrozenDict(identifiers)), None
+        )
+
+    def move(self, source: TableId, target: TableId) -> Cmd[None]:
+        return self.append(source, target) + self.delete_cascade(source)
+
+    def overwrite(self, source: TableId, target: TableId) -> Cmd[None]:
+        nothing = Cmd.from_cmd(lambda: None)
+        recreate = self.exist(target).bind(
+            lambda b: self.delete_cascade(target)
+            + self.create_like(source, target)
+            if b
+            else nothing
+        )
+        return recreate + self.move(source, target)
